@@ -248,12 +248,13 @@ profile_extention: '.' /[\w\-]+/ '.' /[\w-]+/ {
     $return = join('', @item[1..4]) ;
 }
 
-profile_name: 'cross' | 'pkg' | 'stage1'| 'stage2' |
-   'nobiarch' | 'nocheck' | 'nodoc' | 'nogolang' | 'nojava' | 'noperl' | 'nopython' | 'noudeb' |
+profile_name: 'cross' | 'pkg' | 'stage1'| 'stage2' | 'nobiarch' | 'nocheck' | 'nocil' | 'nodoc' |
+    'nogolang' | 'noinsttest' | 'nojava' | 'noperl' | 'nopython' | 'noruby' | 'nolua' | 'noguile' |
+    'noocaml' | 'nowasm' | 'nowindows' | 'noudeb' |
 {
-    my @a = ('cross', 'pkg', 'stage1', 'stage2',
-            'nobiarch', 'nocheck', 'nodoc', 'nogolang', 'nojava', 'noperl', 'nopython', 'noudeb');
-    my ($bad) = split / /, $text;
+    my @a = qw/cross pkg stage1 stage2 nobiarch nocheck nocil nodoc nogolang noinsttest nojava noperl
+               nopython noruby nolua noguile noocaml nowasm nowindows noudeb/;
+    my ($bad) = ($text =~ /^(\w+)/) ;
     $add_error->("Unknown build profile name '$bad'","Expected one of @a") ;
 }
 
