@@ -27,12 +27,13 @@ For details, see L<section 5.7 of Debian policy|https://www.debian.org/doc/debia
       {
         'compute' => {
           'allow_override' => '1',
-          'formula' => 'use Cwd; getcwd =~ m!/([^/]+)$!; $1;',
+          'formula' => 'use Cwd; my $res = getcwd =~ m!/([a-z0-9][a-z0-9+.-]+)$! ? $1 : undef;
+',
           'use_eval' => '1'
         },
-        'description' => 'Source package name. Defaults to the name of the current directory.',
+        'description' => 'Source package name. Defaults to the name of the current directory or undef if the directory name does not match Debian requirements for a source package',
         'mandatory' => '1',
-        'match' => '\\w[\\w+\\-\\.]{1,}',
+        'match' => '^[a-z0-9][a-z0-9+.-]+$',
         'summary' => 'source package name',
         'type' => 'leaf',
         'value_type' => 'uniline'
@@ -85,7 +86,7 @@ If the maintainer\'s name contains a full stop then the whole field will not wor
 Before updating this field, please read L<upgrading-checklist|https://www.debian.org/doc/debian-policy/upgrading-checklist.html>
 to know what changes came with a new policy version number and apply the required changes (if any) to your package.',
         'mandatory' => '1',
-        'match' => '\\d+\\.\\d+\\.\\d+(\\.\\d+)?',
+        'match' => '^\\d+\\.\\d+\\.\\d+(\\.\\d+)?$',
         'summary' => 'Debian policy version number this package complies to',
         'type' => 'leaf',
         'value_type' => 'uniline',
@@ -284,7 +285,9 @@ in Debian developer reference.',
           }
         },
         'class' => 'Config::Model::Dpkg::DependencyList',
-        'description' => 'List of packages that must be installed:
+        'description' => '=pod
+
+List of packages that must be installed:
 
 '.'=over
 
@@ -305,7 +308,15 @@ On the other hand, the list of packages that must be installed to build architec
 
 Including a dependency in this field does not have the exact same effect as including it in both Build-Depends-Arch and Build-Depends-Indep, because the dependency also needs to be satisfied when building the source package.
 
-See also L<deb-src-control|https://manpages.debian.org/unstable/dpkg-dev/deb-src-control.5.en.html> man page',
+See also L<deb-src-control|https://manpages.debian.org/unstable/dpkg-dev/deb-src-control.5.en.html> man page.
+
+This parameter can aslo have restriction formula. For instance:
+
+    Build-Depends: foo (>= 1.0) [i386 arm] <!nocheck> <!cross>, bar
+
+See L<debian wiki|https://wiki.debian.org/BuildProfileSpec#Build-Depends_syntax_extension_.28restriction_formulas.29>
+for more details.
+',
         'duplicates' => 'warn',
         'summary' => 'List of package required during clean or build of architecture-dependent packages',
         'type' => 'list'
@@ -333,12 +344,22 @@ See L<deb-src-control man page|https://manpages.debian.org/unstable/dpkg-dev/deb
           'value_type' => 'uniline'
         },
         'class' => 'Config::Model::Dpkg::DependencyList',
-        'description' => 'List of packages that must be installed to build architecture-independent binaries ("Architecture: all").
+        'description' => '=pod
+
+List of packages that must be installed to build architecture-independent binaries ("Architecture: all").
 
 Technically, these packages must be installed for the following build targets: build, build-indep, binary, and binary-indep.
 See L<build target|https://www.debian.org/doc/debian-policy/#relationships-between-source-and-binary-packages-build-depends-build-depends-indep-build-depends-arch-build-conflicts-build-conflicts-indep-build-conflicts-arch>.
 
-Note that packages required during "clean" phase must be declared in "Build-Depends" field.',
+Note that packages required during "clean" phase must be declared in "Build-Depends" field.
+
+This parameter can aslo have restriction formula. For instance:
+
+    Build-Depends: foo (>= 1.0) [i386 arm] <!nocheck> <!cross>, bar
+
+See L<debian wiki|https://wiki.debian.org/BuildProfileSpec#Build-Depends_syntax_extension_.28restriction_formulas.29>
+for more details.
+',
         'duplicates' => 'warn',
         'summary' => 'List of package required during build of architecture-independent package',
         'type' => 'list'
