@@ -6,6 +6,7 @@ use warnings;
 use 5.20.0;
 use Exporter::Lite;
 use Array::IntSpan;
+use Data::Compare;
 use Path::Tiny;
 use Time::localtime;
 use Carp;
@@ -79,6 +80,8 @@ sub print_copyright ( %args ) {
         next if $paths[0] =~ /\.$/;
 
         $c = "UNKNOWN" unless $c;
+        chomp $c;
+        $c =~ s/\n/\n /g;
         push @out,
             "Files: ", join($whitespace_list_delimiter, @paths )."\n",
             "Copyright: $c\n",
@@ -144,6 +147,7 @@ $default{check} = << 'EOR2' ;
        |scala
        |clj
        |p(l|m)?6?|t|xs|pod6?     # perl5 or perl6
+       |rakumod                  # Raku module file
        |sh
        |php
        |py(|x)
@@ -203,7 +207,8 @@ sub _get_data_from_files  ( %args ) {
             $regexps{$what} = $reg;
         }
 
-        my $lic_cmd = 'licensecheck --encoding utf8 --copyright --machine --shortname-scheme=debian,spdx --recursive '
+        my $lic_cmd = 'licensecheck --encoding utf8 --copyright --machine '
+            .'--lines=300 --shortname-scheme=debian,spdx --recursive '
             . "'--ignore=%s' %s";
         my $cmd = sprintf($lic_cmd, $regexps{ignore}, $current_dir);
         open(my $pipe, "-|", $cmd) or die "Can't open '$cmd' : $!\n";;
@@ -247,10 +252,7 @@ sub _trim_lines ($lines, $regexps) {
     return @kept_lines;
 }
 
-sub __skip_line ($f, $c, $l, $files, $fill_blank_data) {
-    # say "found: $line";
-    return 1 if $files->{$f};       # file already parsed
-
+sub __skip_line ($f, $c, $l, $fill_blank_data) {
     # skip copyright because there's no need to have recursive copyrights
     # skip changelog because it often contains log entries beginning with copyright
     # and it's a collective work by nature.
@@ -279,6 +281,22 @@ sub __skip_line ($f, $c, $l, $files, $fill_blank_data) {
     return 0;
 }
 
+my $__extract_rakudoc_info = sub ($file, $c, $l, $current_dir) {
+    my $rakudoc = $current_dir->child($file);
+
+    if ($rakudoc->is_file) {
+        foreach my $line ($rakudoc->lines_utf8) {
+            if ($line =~ /^=AUTHOR\s+(.*)/) {
+                my $author = $1;
+                # do not override actual info coming from licensecheck
+                $c = $author if $c =~ /no copyright/i;
+                last;
+            }
+        }
+    }
+    return ($c, $l);
+};
+
 my $__extract_rust_info = sub ($file, $c, $l, $current_dir) {
     my $cargotoml = $current_dir->child($file);
 
@@ -322,7 +340,7 @@ my $__extract_nodejs_info = sub ($c_key, $l_key, $file, $c, $l, $current_dir) {
         my @l_data
            = ref $data->{$l_key} eq 'ARRAY' ? $data->{$l_key}->@*
            :                                  $data->{$l_key};
-        return (join("\n ", @c_data) || $c, join(" or ",@l_data) || $l);
+        return (join("\n", @c_data) || $c, join(" or ",@l_data) || $l);
     }
     return ($c, $l);
 };
@@ -334,19 +352,38 @@ my $__extract_json_info = sub ($c_key, $l_key, $file, $c, $l, $current_dir) {
         my $data = from_json($json_file->slurp_utf8);
         my @c_data = ref $data->{$c_key} ? $data->{$c_key}->@* : $data->{$c_key};
         my @l_data = ref $data->{$l_key} ? $data->{$l_key}->@* : $data->{$l_key};
-        return (join("\n ", @c_data) || $c, join(" or ",@l_data) || $l);
+        return (join("\n", @c_data) || $c, join(" or ",@l_data) || $l);
     }
     return ($c, $l);
 };
 
+sub __cleanup_license ($l) {
+    # Found in LICENSE files generated by Dist::Zilla
+    $l =~ s/(l?gpl)_(\d)/uc($1)."-$2"/e;
+    $l =~ s/_/./;
+    $l =~ s!GPL-1\+ and/or GPL-1!GPL-1+!;
+
+    # convert spdx license in Debian style
+    $l =~ s!(L?GPL-\d)\.0!$1!g;
+    $l =~ s!(L?GPL-[\d.]+)-only!$1!g;
+    $l =~ s!(L?GPL-[\d.]+)-or-later!$1+!g;
+
+    # remove identical or'ed license
+    my %lics = map { $_ => 1 } split /\sor\s/, $l;
+    $l = join (' or ', sort keys %lics);
+
+    return $l;
+}
+
 my %override = (
     'Cargo.toml' => $__extract_rust_info,
     'package.json' => sub {$__extract_nodejs_info->('author','license',@_)},
     'META.json' => sub {$__extract_json_info->('author','license',@_)},
     'META6.json' => sub {$__extract_json_info->('authors','license',@_)},
+    'README.rakudoc' => $__extract_rakudoc_info,
 );
 
-sub __refine_line ($f, $c, $l, $current_dir, $files, $fill_blank_data) {
+sub __refine_line ($f, $c, $l, $current_dir, $fill_blank_data) {
     # override license information by scanning for TOML data
     if ($override{$f}) {
         ($c, $l) = $override{$f}->($f, $c, $l, $current_dir);
@@ -357,14 +394,11 @@ sub __refine_line ($f, $c, $l, $current_dir, $files, $fill_blank_data) {
 
     $c = __pack_copyright($c);
 
-    # Found in LICENSE files generated by Dist::Zilla
-    $l =~ s/(l?gpl)_(\d)/uc($1)."-$2"/e;
-    $l =~ s/_/./;
-    $l =~ s!GPL-1\+ and/or GPL-1!GPL-1+!;
+    $l = __cleanup_license($l);
 
     my $fill_blank = __get_fill_blank($fill_blank_data, $f);
     ($c, $l) = __apply_fill_blank($fill_blank, $f, $l, $c);
-    return ($f, $c, $l);
+    return [$c, $l];
 }
 
 sub __store_line_info($f, $c, $l, $id, $files, $copyrights, $no_info_list) {
@@ -384,29 +418,39 @@ sub __store_line_info($f, $c, $l, $id, $files, $copyrights, $no_info_list) {
     return $id;
 }
 
-# option to skip UNKNOWN ?
-# load a file to override some entries ?
-sub scan_files ( %args ) {
-    $quiet = $args{quiet} // 0;
+sub _refine_lines($line_refs,$current_dir, $fill_blank_data) {
+    my $file_info = {};
+    foreach my $ref ($line_refs->@*) {
+        my ($f,$l,$c) = $ref->@*;
 
-    my @line_refs = _get_data_from_files( %args );
-    my $current_dir = $args{from_dir} || path('.');
+        next if __skip_line ($f, $c, $l, $fill_blank_data);
 
-    my $fill_blank_data = __load_fill_blank_data($current_dir);
+        $file_info->{$f} = __refine_line ($f, $c, $l, $current_dir, $fill_blank_data);
+    }
+    return $file_info;
+}
 
+sub _copy_license_in_readme_info ($file_info) {
+    my $new_data = $file_info->{'LICENSE'} // $file_info->{'LICENCE'};
+    if ($new_data and $new_data->[0] !~ /unknown|no-info-found/i) {
+        foreach my $f (sort keys %$file_info) {
+            my ($c,$l) = $file_info->{$f}->@*;
+            # copy license data in README if there's copyright info and no license info
+            if ($f =~ /^README/ and $l =~ /unknown|no-info-found/i and $c !~ /unknown|no-info-found/i) {
+                $file_info->{$f}[1]= $new_data->[1];
+            }
+        }
+    }
+    return;
+}
+
+sub _extract_copyright_info ($file_info) {
     my %copyrights ;
     my $files = {};
     my $id = 1;
     my @no_info_list;
-
-    foreach my $ref (@line_refs) {
-        # say "found: $line";
-        my ($f,$l,$c) = $ref->@*;
-
-        next if __skip_line ($f, $c, $l, $files, $fill_blank_data);
-
-        ($f, $c, $l) = __refine_line ($f, $c, $l, $current_dir, $files, $fill_blank_data);
-
+    foreach my $f (sort keys $file_info->%*) {
+        my ($c,$l) = $file_info->{$f}->@*;
         $id = __store_line_info($f, $c, $l, $id, $files, \%copyrights, \@no_info_list) ;
     }
 
@@ -417,24 +461,43 @@ sub scan_files ( %args ) {
             $copyrights_by_id[$id] = __to_copyright_structure( $c, $l ) ;
         }
     }
+    return (\@copyrights_by_id, $files, \@no_info_list);
+}
 
-    _warn_user_about_problems ($files, $fill_blank_data, @no_info_list);
+# option to skip UNKNOWN ?
+# load a file to override some entries ?
+sub scan_files ( %args ) {
+    $quiet = $args{quiet} // 0;
 
-    my $merged_c_info = __squash_copyrights_years (\@copyrights_by_id) ;
+    my @line_refs = _get_data_from_files( %args );
+    my $current_dir = $args{from_dir} || path('.');
+
+    my $fill_blank_data = __load_fill_blank_data($current_dir);
+
+    my $file_info = _refine_lines(\@line_refs,$current_dir, $fill_blank_data);
+
+    # When README.* has no license info, get it from LICENSE file
+    _copy_license_in_readme_info($file_info);
+
+    my ($copyrights_by_id, $files, $no_info_list) = _extract_copyright_info ($file_info);
+
+    _warn_user_about_problems ($files, $fill_blank_data, $no_info_list);
+
+    my $merged_c_info = __squash_copyrights_years ($copyrights_by_id) ;
 
     # replace the old ids with news ids
     __swap_merged_ids($files, $merged_c_info);
 
     # stop here for update ...
-    return ($files, \@copyrights_by_id) ;
+    return ($files, $copyrights_by_id) ;
 }
 
-sub _warn_user_about_problems ($files, $fill_blank_data, @no_info_list) {
-    if (@no_info_list) {
+sub _warn_user_about_problems ($files, $fill_blank_data, $no_info_list) {
+    if ($no_info_list->@*) {
         my $msg= "The following paths are missing information:\n";
         map {my ($p,@i) = $_->@*;
              $msg .= "- $p: missing ".join(' and ', @i)."\n";
-         } @no_info_list;
+         } $no_info_list->@*;
         $msg .= "You may want to add a line in debian/fill.copyright.blanks.yml\n\n";
         _warn $msg;
     }
@@ -464,9 +527,14 @@ sub __from_copyright_structure ($s) {
 sub __split_copyright ($c) {
     my ($years,$owner) = $c =~ /^(\d\d[\s,\d-]+)(.*)/;
     # say "undef year in $c" unless defined $years;
-    return unless defined $years;
-    my @data = split /(?<=\d)[,\s]+/, $years;
-    return unless defined $owner;
+    if (not defined $years) {
+        # try owner and years in reversed order (works also without year)
+        ($owner,$years) = $c =~ m/(.*?)(\d\d[\s,\d-]+)?$/;
+    }
+
+    return unless $owner;
+
+    my @data = defined $years ? split /(?<=\d)[,\s]+/, $years : ();
     $owner =~ s/^[\s.,-]+|[\s,*-]+$//g;
     return ($owner,@data);
 }
@@ -533,9 +601,10 @@ sub __pack_copyright ($r) {
             }
         }
         $span->consolidate();
-        push @res, $span->get_range_list. ($owner ? ', '. $owner : '');
+        my $range = $span->get_range_list;
+        push @res, join(', ', grep { length $_ } ($range, $owner) );
     }
-    return join("\n ",reverse sort @res);
+    return join("\n",reverse sort @res);
 }
 
 #in each directory, pack files that have the same copyright/license information
@@ -584,7 +653,7 @@ sub __squash_copyrights_years ($copyrights_by_id) {
         #say "id $id: c $c l $l";
         my @owners ;
         my @years ;
-        foreach my $line (split(/\n\s+/,$c)) {
+        foreach my $line (split(/\n/,$c)) {
             my ($owner, @year) = __split_copyright($line);
             next unless defined $owner;
             push @owners, $owner;
@@ -611,7 +680,7 @@ sub __squash_copyrights_years ($copyrights_by_id) {
 
         # store (c) info with coalesced years in new item of $copyrights_by_id
         my $new_id = $copyrights_by_id->@* ;
-        my $new_cop = join("\n ",@squashed_c) ;
+        my $new_cop = join("\n",@squashed_c) ;
         $copyrights_by_id->[$new_id] = __to_copyright_structure( $new_cop , $l );
         #say "created id $new_id with c $new_cop l $l";
         # fill the swap table entry-id -> coaslesces entry-id
@@ -707,11 +776,21 @@ sub __find_main_license_id ($copyrights_by_id, $main_info) {
             $licenses{$lic} = 1 unless $lic =~ /unknown/i;
             $copyrights{$cop} = 1 unless $cop =~ /unknown|no-info-found/i;;
         }
-        push $copyrights_by_id->@*,
-            __to_copyright_structure (
-                join("\n", sort keys %copyrights),
-                join(" or ", sort keys %licenses)
-            );
+
+        my $new_data = __to_copyright_structure (
+            join("\n", reverse sort keys %copyrights),
+            join(" or ", sort keys %licenses)
+        );
+
+        # check whether the new copyright/license already exists
+        # before creating a new one
+        for (my $idx = 0; $idx < $#$copyrights_by_id; $idx++) {
+            if (Compare($copyrights_by_id->[$idx], $new_data)) {
+                return $idx;
+            }
+        }
+
+        push $copyrights_by_id->@*, $new_data;
         $main_license_id = $#$copyrights_by_id;
     }
     return $main_license_id;
